"""
xNode Tests Package

This package contains all tests for the xNode library, including:
- Performance mode tests
- Unit tests for core functionality
- Integration tests
- Error handling tests
"""

__version__ = "1.0.0"
__author__ = "xComBot Team" 